% Reference:
% - Ritschel, T., Ihrke, M., Frisvad, J. R., Coppens, J., Myszkowski, K.,
%   and Seidel, H.-P. Temporal glare: Real-time dynamic simulation of the
%   scattering in the human eye. Computer Graphics Forum 28(2),
%   pp. 183-192, April 2009.
%
% This code tries, qualitatively, to match the Figure by Fry 1991 which
% plots pupil size as a function of time for different glare source
% intensities.
% 
% My expression that mimics pupillary hippus:
% 
% f(t,p) = p + p_max/p * sqrt(1 - p/p_max) * noise(t/p)
% 
% where t is time (in seconds), p is the mean pupil size (in mm) for a
% given glare source intensity, p_max is the maximum pupil size (I've
% used p_max = 9 mm), and noise(...) is a noise function. I chose to use
% improved Perlin noise with three octaves.
% 
% It remains to map glare source intensity to the mean pupil size p.  I
% suspect this involves a log function.
% 
% Jeppe Revall Frisvad, July 2008 (ported to Matlab May 2019) 

pupilsizes = [ 8.5, 7.2, 6.0, 4.5, 4.0 ];
no_of_octaves = 3;
pupilmax = 9;
x = linspace(0, 30, 310);
for i = 1:length(pupilsizes)
    pupilsize = pupilsizes(i);
    y = ones(size(x))*pupilsize;
    amp = pupilmax/pupilsize*sqrt(1 - pupilsize/pupilmax);
    scale = 1/pupilsize;   % scale is the number of blobs in an interval of width 2
    noise_x = x*scale;
    for j = 1:length(x)
        for k = 0:(no_of_octaves-1)
          octave = 2^k;
          nx = noise_x(j)*octave;
          y(j) = y(j) + amp/octave*scnoise([nx + 3.5, nx + 2.1, nx + 7.2] + i*23.2);
        end
    end
    if(i > 1)
        hold on;
    end
    plot(x, y);
end
hold off;
title('Pupillary hippus');
xlabel('time (seconds)');
ylabel('pupil diameter (mm)');
axis([0 30 0 9]);